/**
 * DevExtreme (bundles/__internal/grids/pivot_grid/sortable/module_item_render.js)
 * Version: 22.2.4 (build 23079-0308)
 * Build date: Mon Mar 20 2023
 *
 * Copyright (c) 2012 - 2023 Developer Express Inc. ALL RIGHTS RESERVED
 * Read about DevExtreme licensing here: https://js.devexpress.com/Licensing/
 */
"use strict";
var __importDefault = this && this.__importDefault || function(mod) {
    return mod && mod.__esModule ? mod : {
        default: mod
    }
};
Object.defineProperty(exports, "__esModule", {
    value: true
});
exports.sortableItemRender = void 0;
var size_1 = require("../../../../core/utils/size.js");
var iterator_1 = require("../../../../core/utils/iterator.js");
var renderer_1 = __importDefault(require("../../../../core/renderer.js"));
var const_1 = require("./const");

function getTreeViewItem($sourceItem) {
    return $sourceItem.clone().addClass(const_1.SortableConst.classes.areaBox).css("width", parseFloat(size_1.getOuterWidth($sourceItem)))
}

function getAreaBoxItemArray($sourceItem, target) {
    var $itemArray = $sourceItem.clone();
    if (target === const_1.SortableConst.targets.drag) {
        iterator_1.each($sourceItem, (function(idx, sourceItem) {
            var width = parseFloat(size_1.getOuterWidth(sourceItem));
            $itemArray.eq(idx).css("width", width)
        }))
    }
    return $itemArray
}

function getDefaultItem($sourceItem) {
    return renderer_1.default("<div>").addClass(const_1.SortableConst.classes.areaField).addClass(const_1.SortableConst.classes.areaBox).text($sourceItem.text())
}

function getItemArray($sourceItem, target) {
    var isAreaBox = $sourceItem.hasClass(const_1.SortableConst.classes.areaBox);
    var isTreeList = $sourceItem.attr(const_1.SortableConst.attrs.treeViewItem);
    if (isAreaBox) {
        return getAreaBoxItemArray($sourceItem, target)
    }
    if (isTreeList) {
        return getTreeViewItem($sourceItem)
    }
    return getDefaultItem($sourceItem)
}

function wrapItemsInFieldsContainer($itemArray) {
    var $wrappedTmpContainer = renderer_1.default("<div>");
    iterator_1.each($itemArray, (function(_, item) {
        var $wrappedItem = renderer_1.default("<div>").addClass(const_1.SortableConst.classes.fieldsContainer).addClass(const_1.SortableConst.classes.widget).append(renderer_1.default(item));
        $wrappedTmpContainer.append($wrappedItem)
    }));
    return $wrappedTmpContainer.children()
}

function sortableItemRender($sourceItem, target) {
    var $itemArray = getItemArray($sourceItem, target);
    return target === const_1.SortableConst.targets.drag ? wrapItemsInFieldsContainer($itemArray) : $itemArray
}
exports.sortableItemRender = sortableItemRender;
