/**
 * DevExtreme (esm/__internal/grids/pivot_grid/sortable/module_item_render.js)
 * Version: 22.2.4 (build 23079-0308)
 * Build date: Mon Mar 20 2023
 *
 * Copyright (c) 2012 - 2023 Developer Express Inc. ALL RIGHTS RESERVED
 * Read about DevExtreme licensing here: https://js.devexpress.com/Licensing/
 */
import {
    getOuterWidth
} from "../../../../core/utils/size.js";
import {
    each
} from "../../../../core/utils/iterator.js";
import $ from "../../../../core/renderer.js";
import {
    SortableConst
} from "./const";

function getTreeViewItem($sourceItem) {
    return $sourceItem.clone().addClass(SortableConst.classes.areaBox).css("width", parseFloat(getOuterWidth($sourceItem)))
}

function getAreaBoxItemArray($sourceItem, target) {
    const $itemArray = $sourceItem.clone();
    if (target === SortableConst.targets.drag) {
        each($sourceItem, (idx, sourceItem) => {
            const width = parseFloat(getOuterWidth(sourceItem));
            $itemArray.eq(idx).css("width", width)
        })
    }
    return $itemArray
}

function getDefaultItem($sourceItem) {
    return $("<div>").addClass(SortableConst.classes.areaField).addClass(SortableConst.classes.areaBox).text($sourceItem.text())
}

function getItemArray($sourceItem, target) {
    const isAreaBox = $sourceItem.hasClass(SortableConst.classes.areaBox);
    const isTreeList = $sourceItem.attr(SortableConst.attrs.treeViewItem);
    if (isAreaBox) {
        return getAreaBoxItemArray($sourceItem, target)
    }
    if (isTreeList) {
        return getTreeViewItem($sourceItem)
    }
    return getDefaultItem($sourceItem)
}

function wrapItemsInFieldsContainer($itemArray) {
    const $wrappedTmpContainer = $("<div>");
    each($itemArray, (_, item) => {
        const $wrappedItem = $("<div>").addClass(SortableConst.classes.fieldsContainer).addClass(SortableConst.classes.widget).append($(item));
        $wrappedTmpContainer.append($wrappedItem)
    });
    return $wrappedTmpContainer.children()
}
export function sortableItemRender($sourceItem, target) {
    const $itemArray = getItemArray($sourceItem, target);
    return target === SortableConst.targets.drag ? wrapItemsInFieldsContainer($itemArray) : $itemArray
}
